unit rkCustomRichEditor;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Graphics, Controls, ComCtrls, ExtCtrls, StdCtrls,
  Dialogs, RichMemo;

type

  { TfrCustomRichEditor }

  TfrCustomRichEditor = class(TFrame)
    cboFont: TComboBox;
    cboSize: TComboBox;
    imgMain: TImageList;
    lblSize: TLabel;
    lblFont: TLabel;
    pnlEditor: TPanel;
    pnlSize: TPanel;
    pnlFont: TPanel;
    dlgOpen: TOpenDialog;
    dlgSave: TSaveDialog;
    sbMain: TStatusBar;
    tbMain: TToolBar;
    btnOpen: TToolButton;
    btnSave: TToolButton;
    btnBold: TToolButton;
    btnItal: TToolButton;
    btnUndl: TToolButton;
    btnLink: TToolButton;
    procedure btnLinkClick(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure EditLinkAction(Sender: TObject; ALinkAction: TLinkAction;
      const Info: TLinkMouseInfo; LinkStart, LinkLen: Integer);
    procedure StyleBtnsClick(Sender: TObject);
    procedure EditSelectionChange(Sender: TObject);
  private
    FSaved: Boolean;
    FFileName: string;
    FFontParams: TFontParams;
    procedure UpdateToolBar;
    procedure SetStyle(AStyle: TFontStyle);
  protected
    FRichEdit: TCustomRichMemo;
  public
    constructor Create(TheOwner: TComponent); override;
  end;

implementation

uses LCLIntf, RichMemoUtils;

{$R *.lfm}

function GetFileName(const ADialog: TFileDialog; var AFileName: string): Boolean;
begin
  Result := False;
  with ADialog do
  begin
    if (AFileName > '') then
      FileName := AFileName
    else
      InitialDir := GetCurrentDir;
    Result := Execute;
    if Result then AFileName := FileName;
  end;
end;


{ TfrCustomRichEditor }

constructor TfrCustomRichEditor.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  btnBold.Tag := Ord(fsBold);
  btnItal.Tag := Ord(fsItalic);
  btnUndl.Tag := Ord(fsUnderLine);
  cboFont.Items.Assign(Screen.Fonts);
end;

procedure TfrCustomRichEditor.btnOpenClick(Sender: TObject);
begin
  if GetFileName(dlgOpen, FFileName) then
    LoadRTFFile(FRichEdit, FFileName);
end;

procedure TfrCustomRichEditor.btnSaveClick(Sender: TObject);
begin
  if GetFileName(dlgSave, FFileName) then
    SaveRTFFile(FRichEdit, FFileName);
end;

procedure TfrCustomRichEditor.StyleBtnsClick(Sender: TObject);
begin
  SetStyle(TFontStyle(TToolButton(Sender).Tag));
end;

procedure TfrCustomRichEditor.btnLinkClick(Sender: TObject);
begin
  with FRichEdit do
    SetLink(SelStart, SelLength, not IsLink(SelStart));
  UpdateToolBar;
end;

procedure TfrCustomRichEditor.EditLinkAction(Sender: TObject; ALinkAction: TLinkAction;
                         const Info: TLinkMouseInfo; LinkStart, LinkLen: Integer);
begin
  if (Info.Button = mbLeft) then
    OpenURL(FRichEdit.GetText(LinkStart, LinkLen));
end;

procedure TfrCustomRichEditor.EditSelectionChange(Sender: TObject);
begin
  FRichEdit.GetTextAttributes(FRichEdit.SelStart, FFontParams);
  UpdateToolBar;
end;

procedure TfrCustomRichEditor.UpdateToolBar;
begin
  cboFont.Caption := FFontParams.Name;
  cboSize.Caption := IntToStr(FFontParams.Size);
  btnBold.Down := (fsBold in FFontParams.Style);
  btnItal.Down := (fsItalic in FFontParams.Style);
  btnLink.Down := FRichEdit.IsLink(FRichEdit.SelStart);
  btnUndl.Down := not btnLink.Down and (fsUnderline in FFontParams.Style);
end;

procedure TfrCustomRichEditor.SetStyle(AStyle: TFontStyle);
begin
  if (AStyle in FFontParams.Style) then
    FFontParams.Style := FFontParams.Style - [AStyle]
  else
    FFontParams.Style := FFontParams.Style + [AStyle];
  FRichEdit.SetTextAttributes(FRichEdit.SelStart, FRichEdit.SelLength, FFontParams);
  UpdateToolbar;
end;

initialization
  RegisterClass(TfrCustomRichEditor);

end.

